Imports T = System.Double
Imports madrat.Utility.Functional

Namespace Calculate.Expressions

    ' HACK: The compiler is buggy when it comes to type inference through generic delegates.
    ' There are two ways to fix this; one is to use the very long, very complicated explicit
    ' instantiation of the delegates (using 'New'). The other, shorter, is to explicitly
    ' specify the type parameters used. (I used the second one.)

    Public Class Values : Inherits List(Of T)

        Public Sub New()
        End Sub

        Public Sub New(ByVal collection As IEnumerable(Of T))
            MyBase.New(collection)
        End Sub

        Private Function tos(Of T)(ByVal obj As T) As String
            Return obj.ToString()
        End Function

        ''' <remarks>I *yearn* for C#'s anonymous delegates!</remarks>
        Public Overrides Function ToString() As String
            Return FoldLeft(Of String, String)(Map(Of T, String)(Me, AddressOf DToS), "", AddressOf CatComma)
        End Function

        Public Shared Widening Operator CType(ByVal values As Double()) As Values
            Return New Values(values)
        End Operator

        Public Shared Operator -(ByVal lhs As Values) As Values
            Return New Values(Map(Of T, T)(lhs, AddressOf Arithmetics.op_UnaryNegation))
        End Operator

        Public Shared Operator +(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_Addition))
        End Operator

        Public Shared Operator -(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_Subtraction))
        End Operator

        Public Shared Operator *(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_Multiply))
        End Operator

        Public Shared Operator /(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_Division))
        End Operator

        Public Shared Operator /(ByVal lhs As Double, ByVal rhs As Values) As Values
            Return New Values(MapValueLeft(Of T, T, T)(rhs, lhs, AddressOf Arithmetics.op_Division))
        End Operator

        Public Shared Operator \(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_IntegerDivision))
        End Operator

        Public Shared Operator Mod(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_Modulus))
        End Operator

        Public Shared Operator ^(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_Exponent))
        End Operator

        ' HACK: '<<' requires a second parameter of type 'Integer' which I had to work around.
        Public Shared Function op_LeftShift(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_LeftShift))
        End Function

        ' HACK: '>>' requires a second parameter of type 'Integer' which I had to work around.
        Public Shared Function op_RightShift(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_RightShift))
        End Function

        Public Shared Operator Not(ByVal lhs As Values) As Values
            Return New Values(Map(Of T, T)(lhs, AddressOf Arithmetics.op_OnesComplement))
        End Operator

        Public Shared Operator And(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_BitwiseAnd))
        End Operator

        Public Shared Operator Or(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_BitwiseOr))
        End Operator

        Public Shared Operator Xor(ByVal lhs As Values, ByVal rhs As Values) As Values
            Return New Values(ZipWith(Of T, T, T)(lhs, rhs, AddressOf Arithmetics.op_ExclusiveOr))
        End Operator

#Region " Private Helper Functions "

        Private Function DToS(ByVal d As Double) As String
            Return d.ToString()
        End Function

        Private Function CatComma(ByVal a As String, ByVal b As String) As String
            If a = "" Then Return b
            Return a & ", " & b
        End Function

#End Region

    End Class

End Namespace