Imports madrat.Common
Imports madrat.Calculate
Imports madrat.Calculate.Expressions

Namespace Utility

    Class Display

        Public Shared Sub FormatResult(ByVal vs As Values)
            Reader.State = InputReader.ActionState.Output

            For Each v As Double In vs
                ' Look whether the user has aborted the output:
                If Reader.State = InputReader.ActionState.Input Then Exit For

                Select Case Reader.Precision
                    Case InputReader.DisplayPrecision.Low
                        Console.Write("{0,-8:0.####}", v, ControlChars.Tab)
                    Case InputReader.DisplayPrecision.High
                        Console.Write("{0,-20:G}", v, ControlChars.Tab)
                End Select
            Next

            Console.WriteLine()
        End Sub

        Public Shared Sub WrapTextOut( _
            ByVal text As String, _
            Optional ByVal length As Integer = 79, _
            Optional ByVal indent As Integer = 0 _
        )
            Dim lines As String() = [String].SplitLines(text, length - indent)
            Dim indent_string As String = New String(" "c, indent)
            Dim out As New List(Of String)()

            For Each line As String In lines
                out.Add(indent_string & line)
            Next

            LongTextOut(out.ToArray())
        End Sub

        Public Shared Sub WrapTextOut(ByVal text As String, ByVal ParamArray obj As Object())
            WrapTextOut(String.Format(text, obj))
        End Sub

        ''' <summary>
        ''' Prints a long text to the console, waiting for a user prompt every time the text fills
        ''' a whole console screen page before continuing to print the text.
        ''' </summary>
        ''' <param name="text">The text to print.</param>
        Public Shared Sub LongTextOut(ByVal text As String)
            LongTextOut([String].SplitLines(text, 79))
        End Sub

        ''' <summary>
        ''' Prints a long text to the console, waiting for a user prompt every time the text fills
        ''' a whole console screen page before continuing to print the text.
        ''' </summary>
        ''' <param name="text">The text to print.</param>
        Public Shared Sub LongTextOut(ByVal text As IO.StringWriter)
            LongTextOut([String].Split(text.ToString(), ControlChars.NewLine))
        End Sub

        ''' <summary>
        ''' Prints a long text to the console, waiting for a user prompt every time the text fills
        ''' a whole console screen page before continuing to print the text.
        ''' </summary>
        ''' <param name="text">The text to print.</param>
        Public Shared Sub LongTextOut(ByVal text As String())
            Dim lines As String() = text

            Array.Reverse(lines)
            Dim helptext As New Stack(Of String)(lines)

            Do While helptext.Count > 0
                For i As Integer = 1 To Console.WindowHeight - 2
                    If helptext.Count = 0 Then Exit Do
                    Console.WriteLine(helptext.Pop())
                Next

                If helptext.Count = 0 Then Exit Do

                Console.Write("Please press a key to continue ...")
                Console.ReadKey()
                Console.WriteLine()
            Loop
        End Sub

        ''' <summary>
        ''' Wraps a long text without line breaks into multiple lines. An optional indentation may be given.
        ''' </summary>
        ''' <param name="text">The text to wrap.</param>
        ''' <param name="length">The length of a line.</param>
        ''' <param name="indent">The indentation. Optional. Standard value is 0.</param>
        ''' <returns>Returns the wrapped text.</returns>
        Public Shared Function WrapLongText( _
            ByVal text As String, _
            Optional ByVal length As Integer = 79, _
            Optional ByVal indent As Integer = 0 _
        ) As String
            Dim ret As New Text.StringBuilder()

            For Each paragraph As String In [String].Split(text, ControlChars.NewLine)
                Dim lines As String() = [String].SplitLines(paragraph, length - indent)
                Dim indent_string As String = New String(" "c, indent)

                For Each line As String In lines
                    ret.Append(indent_string)
                    ret.AppendLine(line)
                Next
            Next

            Return ret.ToString()
        End Function

    End Class

End Namespace